// clang-format off

/*

 * mirp | MatterLinux pool mirror tool
 * MatterLinux 2023-2024 (https://matterlinux.xyz)

 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.

*/

// clang-format on

#include <errno.h>
#include <libmp/all.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include "intl.h"
#include "log.h"

bool sync_callback(
    lm_ctx_t *ctx, lm_pool_t *pool, lm_ctx_sync_state_t state, size_t current, size_t total, void *data) {
  if (state == SYNC_INFO_FAIL || state == SYNC_LIST_FAIL) {
    error(_("Failed to sync %s: %s"), pool->name, lm_strerror());
    return false;
  }

  return true;
}

int main(int argc, char *argv[]) {
  if (argc != 4) {
    error(_("Usage: %s <name> <directory> <url>"), argv[0]);
    return EXIT_FAILURE;
  }

  char *pool_name = argv[1];
  char *pool_dir  = argv[2];
  char *pool_url  = argv[3];

  ssize_t       name_len = strlen(pool_name);
  char          tmpdir[name_len + 15];
  lm_pool_t    *pool = NULL;
  lm_pkg_t     *pkg  = NULL;
  lm_ctx_list_t list;
  bool          ret = false;
  lm_ctx_t      ctx;

  sprintf(tmpdir, "/tmp/mirp_%s", pool_name);

  if (!lm_ctx_new(&ctx, NULL, tmpdir, NULL)) {
    switch (lm_error()) {
    case LM_ERR_CtxTempFail:
      error(_("Failed to access the temp directory: %s"), tmpdir);
      break;

    default:
      error(_("Failed to initialize the libmp context: %s"), lm_strerror());
      break;
    }
    goto end;
  }

  if (mkdir(pool_dir, 0755) < 0 && errno != EEXIST) {
    error(_("Failed to create the pool directory: %s"), pool_dir);
    goto end;
  }

  if (access(pool_dir, W_OK) != 0) {
    error(_("Failed to access to the pool directory: %s"), pool_dir);
    goto end;
  }

  if ((pool = lm_ctx_pool_add(&ctx, pool_name, pool_url, pool_dir)) == NULL) {
    error(_("Failed to load the pool: %s"), lm_strerror());
    goto end;
  }

  if (lm_ctx_sync(&ctx, true, sync_callback, NULL) != 1) {
    error(_("Pool is not avaliable"));
    goto end;
  }

  if (NULL == lm_ctx_list(&ctx, &list)) {
    error(_("Failed to obtain the package list: %s"), lm_strerror());
    goto end;
  }

  info(_("Synced %s, downloading packages"), pool->name);

  while ((pkg = lm_ctx_list_next(&list)) != NULL) {
    if (!lm_ctx_download(&ctx, pkg, NULL, NULL)) {
      error(_("Failed to download the package: %s (%s)"), pkg->data.name, pkg->data.version);
      goto end;
    }
    info(_("Downloaded the package: %s (%s)"), pkg->data.name, pkg->data.version);
  }

  lm_ctx_list_free(&list);
  info(_("Mirror is ready"));
end:
  lm_ctx_temp_clear(&ctx);
  lm_ctx_free(&ctx);
  return ret ? EXIT_SUCCESS : EXIT_FAILURE;
}
